/***************
adevs, a library supporting discrete event simulation
Copyright (C) 2001 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
/**
Thread safe version of the atomic model simulator.
Also includes algorithm for doing conservative simulation.
The algorithm used is based on the conservative algorithm
outlined in "Time Management and Interoperability in Distributed
Discrete Event Simulation", my Master's thesis from the University
of Arizona.  Electronic copies are available from 
www.acims.arizona.edu
*/
class pDEVSsimulator: private psimulator, public DEVSsimulator {

public:		
		pDEVSsimulator ();
// Initiates parallel execution of computeNextOutput
		virtual void computeNextOutput (stime_t t);
// Initiates parallel execution of computeIO
		virtual void computeIO (stime_t t);
// Initiates parallel execution of computeTransition
		virtual void computeTransition (stime_t t);
		virtual stime_t timeNext ();
		virtual stime_t sigma ();
// Join with parallel threads
		virtual void wait ();
// Destructor
		virtual ~pDEVSsimulator ();

private:
// Mutex for preventing concurrent access to the simulator
		Mutex mtx;
// Thread for parallel processing
		Thread* thr;
// Variable used to pass time arguments to concurrent functions
		stime_t t_thread;
// Functions and methods that implement the conservative algorithm
		friend void sim_compute_next_output (void* arg);
		friend void sim_compute_io (void* arg);
		friend void sim_compute_transition (void* arg);
		friend void do_conservative_execution (pDEVSsimulator* sim);
		void _computeNextOutput (stime_t t);
		void _computeIO (stime_t t);
		void _computeTransition (stime_t t);  

};
