/***************
adevs, a library supporting discrete event simulation
Copyright (C) 1999, 2000, 2001 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
/**
A binary search tree to support rapid detection of duplicate
objects in an indexed object.  The bst is not intended for use
by classes outside of the container hierarchy.
*/
class bst {
	public:		
		bst ();	
		void insert(void* value);
		/// Returns true if the value is in the tree, false otherwise.
		bool contains(const void* value) const;
		/// Removes all elements from the tree
		void clear_tree();
		/// Returns number of allocated nodes (for testing)
		inline int _bstnode_count() const { return __bstnode_count; }
		/// Deletes the structure only
		~bst();
	private:	
		// A node in the tree
		class _bstnode {
			public:
				inline _bstnode() {}
				void* value;
				_bstnode* left;
				_bstnode* right;
				_bstnode* next;
		};
		_bstnode* root;
		_bstnode* freelist;
		int __bstnode_count;

		_bstnode* makeNode(void* value);
		void freeNode(_bstnode* node);

};
