/***************
Copyright (C) 2001 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/

/**
The rv class provides a random variable based on a selectable
implementation.  By default, this implementation is the Mersenne
Twister (see mtrand).
*/

class rv {

public:
/// Create a random variable with the default implementation.
		rv (unsigned long seed = 1);
/**
Create a random variable with the desired implementation.  The
implementation class is adopted by the rv.
*/
		rv (random_seq* rand);
/// Copy constructor relies on copy method of underlying stream.
		rv (const rv& src);
/// Assignment operator relies on copy method of underlying stream.
		const rv& operator= (const rv& src);
/// See the random number generator implementation
		void set_seed (unsigned long seed);
/// Sample a triangular distribution with (a, b, c)
		double triangular (double a, double b, double c);
/// Sample a uniform distribution in the range [a, b]
		double uniform (double a, double b);
/// Sample an exponential distribution with mean m
		double exp (double m);
/**
Sample a normally distributed random variable with mean m and 
standard deviation s.
*/
		double normal (double m, double s);
/// Destructor
		~rv ();

private:	
		random_seq* _impl;

};

