/***************
adevs, a library supporting discrete event simulation
Copyright (C) 1999, 2000 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
/**
An unsigned integer type wrapped in an object.
*/

class uintg: public object {

public:	unsigned value;

/// Creates an uintg with initial value of 0.
inline uintg ():object () { value = 0; }
/// Copy constructor. 
inline uintg (const uintg &src):object (src) { value = src.value; }
/// Creates an uintg with the indicated initial value.
inline uintg (unsigned d):object () { value = d; }

inline uintg operator+ (const uintg &b) const { return uintg (value + b.value); }
inline uintg operator+ (const unsigned b) const { return uintg (value + b); }

inline uintg operator- (const uintg &b) const { return uintg (value - b.value); }
inline uintg operator- (const unsigned b) const { return uintg (value - b); }

inline uintg operator* (const uintg &b) const { return uintg (value * b.value); }
inline uintg operator* (const unsigned b) const { return uintg (value * b); }

inline uintg operator/ (const uintg &b) const { return uintg (value / b.value); }
inline uintg operator/ (const unsigned b) const { return uintg (value / b); }

/// Mod operator
inline uintg operator% (const uintg& b) const { return uintg (value % b.value); }
/// Mod operator
inline uintg operator% (const unsigned b) const { return uintg (value % b); }

inline bool operator== (const uintg &b) const { return value == b.value; }
inline bool operator== (const unsigned b) const { return value == b; }
inline bool operator!= (const uintg &b) const { return value != b.value; }
inline bool operator!= (const unsigned b) const { return value != b; }

inline bool operator>= (const unsigned b) const { return value >= b; }
inline bool operator>= (const uintg& b) const { return value >= b.value; }

inline bool operator> (const unsigned b) const { return value > b; }
inline bool operator> (const uintg& b) const { return value > b.value; }

inline bool operator<= (const unsigned b) const { return value <= b; }
inline bool operator<= (const uintg& b) const { return value <= b.value; }

inline bool operator< (const unsigned b) const { return value < b; }
inline bool operator< (const uintg& b) const { return value < b.value; }

inline const uintg& operator= (const uintg &b) { value = b.value; return *this; }
inline const uintg& operator= (const unsigned b) { value = b; return *this; }

inline const uintg& operator+= (const uintg &b) { value += b.value; return *this; }
inline const uintg& operator+= (const unsigned b) { value += b; return *this; }

inline const uintg& operator-= (const uintg &b) { value -= b.value; return *this; }
inline const uintg& operator-= (const unsigned b) { value -= b; return *this; }

inline const uintg& operator*= (const uintg &b) { value *= b.value; return *this; }
inline const uintg& operator*= (const unsigned b) { value *= b; return *this; }

inline const uintg& operator/= (const uintg &b) { value /= b.value; return *this; }
inline const uintg& operator/= (const unsigned b) { value /= b; return *this; }

inline const uintg& operator%= (const uintg &b) { value %= b.value; return *this; }
inline const uintg& operator%= (const unsigned b) { value %= b; return *this; }

inline const uintg& operator++ () { value++; return *this; }
inline const uintg& operator-- () { value--; return *this; }

virtual bool operator== (const object &obj) const;
virtual bool operator!= (const object &obj) const;
virtual unsigned hashcode () const;

virtual object* clone () const;

~uintg ();

}; // end of class uintg

