/***************
adevs, a library supporting discrete event simulation
Copyright (C) 2001 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#include <adevs/adevs.h>
using namespace adevs;

void* adevs::exec_func (void* arg) {

Thread* self = (Thread*)arg;

while (true) {
   // Wait until we are given a function to execute
   self->mtx.lock ();
   // If a function is available, run it with the supplied arguement
   if (self->func != NULL) {
      self->func (self->arg);
      }
   // Otherwise, the thread will exit
   else {
      self->mtx.unlock ();
      return NULL;
      }
   }

} 

Thread::Thread():object() {

// No intial functions
func = NULL;
arg = NULL;
// Indicate that thread should be idle
mtx.lock ();
// Create the POSIX thread whose execution stars in the exec_func function
#ifdef PARALLEL_POSIX
pthread_create (&thr, NULL, exec_func, (void*)this);
#endif

}

void Thread::run (void (*_func)(void*), void* _arg) {

// Get the function pointer and argument to be executed
func = _func;
arg = _arg;
// Indicate to the POSIX thread that a function is available to be run
mtx.unlock ();
#ifndef PARALLEL_POSIX
if (func != NULL) func (arg);
#endif

}

Thread::~Thread () {

// Setting the func to NULL indicates that thread should exit (see exec_func)
func = NULL;
arg = NULL;
// Tell the thread to check that func == NULL and then terminate
mtx.unlock ();
// Wait for thread to terminate, then finish deleting self
#ifdef PARALLEL_POSIX
pthread_join (thr, &arg);
#endif

}
