/***************
adevs, a library supporting discrete event simulation
Copyright (C) 1999, 2000, 2001 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#include <adevs/adevs.h>
using namespace adevs;

bst::_bstnode* bst::makeNode(void* value) {
	bst::_bstnode* item;
	if (freelist != NULL) {
		item = freelist;
		freelist = item->next;
		}
	else {
		item = new _bstnode();
		__bstnode_count++;
		}
	item->right = item->left = item->next = NULL;
	item->value = value;
	return item;
}

void bst::freeNode(bst::_bstnode* item) {
	if (item->left != NULL) freeNode (item->left);
	if (item->right != NULL) freeNode (item->right);
	item->next = freelist;
	freelist = item;
}
  
bst::bst() {
	__bstnode_count = 0;
	freelist = root = NULL;
}

void bst::insert(void* value) {
	if (root == NULL) {
		root = makeNode (value);
		return;
		}
	_bstnode* pos = root;
	_bstnode* last = pos;
	bool left = false;
	while (pos != NULL) {
		last = pos;
		if (pos->value > value) { left = true; pos = pos->left; }
		else if (pos->value < value) { left = false; pos = pos->right; }
		else return;
		}
	if (left) last->left = makeNode (value);
	else last->right = makeNode (value);
}

bool bst::contains(const void* value) const {
	_bstnode* pos = root;
	while (pos != NULL) {
		if (pos->value > value) pos = pos->left; 
		else if (pos->value < value) pos = pos->right; 
		else return true;
		}
	return false;
}

void bst::clear_tree() {
	if (root != NULL) freeNode (root);
	root = NULL;
}

bst::~bst() {
	// Put all nodes back on the free list
	clear_tree ();
	// Delete the freelist
	_bstnode* to_delete;
	while (freelist != NULL) {
		to_delete = freelist;
		freelist = freelist->next;
		delete to_delete;
		}
}
