/***************
adevs, a library supporting discrete event simulation
Copyright (C) 1999, 2000, 2001 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#include <adevs/adevs.h>
using namespace adevs;

digraph::digraph():
object() {
	this->grow = DEFAULT_CONTAINER_GROWTH;
	this->capacity = DEFAULT_CONTAINER_CAPACITY;
	vertice_count = 0;
	edge_count = 0;
	lists = new hashtable(capacity,grow);
}

digraph::digraph(unsigned capacity, unsigned grow):
object() {
	this->grow = grow;
	this->capacity = capacity;
	vertice_count = 0;
	edge_count = 0;
	lists = new hashtable(capacity, grow);
}

bool digraph::adjacent(const object &u, const object &v) const {
	indexed* l = (indexed*)lists->find(u);
	unsigned size = l->getSize();
	for (unsigned i = 0; i < size; i++)
		if (*(l->getObj(i)) == v) return true;
	return false;
}

bool digraph::hasVertice(const object &u) const {
	return (lists->find(u) != NULL);
}

const indexed* digraph::neighbors(const object &u) const {
	return (indexed*)lists->find(u);
}

void digraph::addVertice(object *v) {
	vertice_count++;
	lists->insert(v,new indexed(capacity,grow));
}

void digraph::addEdge(const object &u, const object &v) {
	object **keys = lists->getKeys();
	indexed* l = (indexed*)lists->find(u);
	for (unsigned i = 0; i < vertice_count; i++) {
		if (*(keys[i]) == v) {
			l->add(keys[i]);
			edge_count++;
			break;
			}
		}
	delete [] keys;
}

object* digraph::removeVertice(const object &v) {
	object *result;
	object **keys = lists->getKeys();
	indexed *l;
	for (unsigned i = 0; i < vertice_count; i++) {
		if (*(keys[i]) == v) result = keys[i];
		else {
			l = (indexed*)lists->find(*(keys[i]));
			for (unsigned j = 0; j < l->getSize (); j++) {
				if (*(l->getObj(j)) == v) {
					l->remove(j);
					j--;
					edge_count--;
					}
				}
			}
		}
	l = (indexed*)lists->find(*result);
	lists->remove(*result);
	edge_count -= l->getSize();
	delete l;
	delete [] keys;
	vertice_count--;
	return result;
}

void digraph::removeEdge(const object &u, const object &v) {
	indexed* l = (indexed*)lists->find(u);
	for (unsigned i = 0; i < l->getSize (); i++) {
		if (*(l->getObj(i)) == v) {
			l->remove(i);
			edge_count--;
			return;
			}
		}
}

digraph::~digraph() {
	object** keys = lists->getKeys();
	for (unsigned i = 0; i < vertice_count; i++) {
		delete lists->find(*(keys[i]));
		lists->remove(*(keys[i]));
		}
	delete [] keys;
	delete lists;
}
