/***************
adevs, a library supporting discrete event simulation
Copyright (C) 1999, 2000 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#include <adevs/adevs.h>
using namespace adevs;

void set::add(object* obj) {
	if (instanceCount(*obj) == 0) 
		collection::add(obj);
}

set* set::operator-(const set &b) const {
	unsigned size = getSize();
	set *result = new set(size);
	for (unsigned i = 0; i < size; i++) {
		if (b.instanceCount (*(getObj(i))) == 0)
			result->add (getObj(i));
		}
	return result;
}

set& set::operator-=(const set &b) {
	unsigned size = b.getSize();
	for (unsigned i = 0; i < size; i++) {
		if (instanceCount(*(b[i])) > 0)
			remove (indexOf((*b[i])));
		}
	return *this;
}

set* set::Union(const set &b) const {
	unsigned size = b.getSize();
	set* result = new set(getSize() + b.getSize());
	for (unsigned i = 0; i < size; i++)
		result->add(b[i]);
	size = getSize();
	for (unsigned i = 0; i < size; i++)
		result->add(getObj(i));
	return result;
}

set& set::assignUnion(const set &b) {
	unsigned size = b.getSize();
	for (unsigned i = 0; i < size; i++)
		add(b[i]);
	return *this;
}

set& set::operator*=(const set &b) {
	for (unsigned i = 0; i < getSize (); i++) {
		if (b.instanceCount(*(getObj(i))) == 0) {
			remove(i);
			i--;
			}
		}
	return *this;
}

set* set::operator*(const set &b) const {
	unsigned size;
	set* result;
	if (getSize() < b.getSize()) {
		size = getSize();
		result = new set(size);
		for (unsigned i = 0; i < size; i++) {
			if (b.instanceCount(*(getObj(i))) > 0)
				result->add(getObj(i));
			}
		}
	else {
		size = b.getSize();
		result = new set(size);
		for (unsigned i = 0; i < size; i++) {
			if (this->instanceCount(*(b[i])) > 0)
				result->add (b[i]);
			}
		}
	return result;
}

set& set::operator+=(const set &b) {
	set *y = b - *this;
	*this -= (b);
	assignUnion (*y);
	delete y;
	return *this;
}

set* set::operator+(const set &b) const {
	set *x = b - *this;
	set *y = *this - b;
	x->assignUnion (*y);
	delete y;
	return x;
}

bool set::operator==(const set &b) const {
	if (getSize() != b.getSize()) return false;
	unsigned size = getSize();
	for (unsigned i = 0; i < size; i++) {
		if (instanceCount(*(b[i])) == 0) return false;
		if (b.instanceCount (*(getObj(i))) == 0) return false;
		}
	return true;
}

bool set::operator!=(const set &b) const {
	return !(*this == b);

}

unsigned set::hashcode() const {
	return getSize();
}
