/***********
Copyright (C) 1999, 2000  James Nutaro

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#include "message.h"
#include "prog.h"
#include <iostream>
using namespace adevs;

const port_t prog::in = 0;
const port_t prog::out = 1;

prog::prog (unsigned pid, stime_t max_interval, unsigned messages)
:atomic ("prog") {

m_out = NULL;
this->pid = pid;
this->to_send = messages;
this->max_interval = max_interval;
addInputPort (in);
addOutputPort (out);

}

void prog::reset () {

if (m_out != NULL) delete m_out;
m_out = NULL;
le.empty ();
sent = 0;
hold (0);

}

void prog::delta_int () {

m_out = NULL;

// if quota of messages has not been reached, output a new message
if (sent < to_send) {
   m_out = new message (pid, sent);
   sent++;
   hold ((double)random () / (double)RAND_MAX * max_interval);
   }

else passivate ();

}


void prog::delta_ext () {

// add messages from the incoming message queue to the execution list
bag* values = inputOnPort (in);
for (unsigned i = 0; i < values->getSize (); i++) {
   queue* q = (queue*)values->getObj (i);
   while (q->getSize () > 0) le.add (q->remove ());
   }

}

void prog::delta_conf () {

delta_int ();
delta_ext ();

}

void prog::output_func () {

if (m_out != NULL) output (out, m_out);

}

prog::~prog () {

if (m_out != NULL) delete m_out;
le.empty ();

}
