/***********
Copyright (C) 1999, 2000  James Nutaro

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/

/**

A program running in a logical process.  Generates events at random intervals.
The execution list represents the program being executed.  Operations are
added to the list in the order that they are received.

*/

class prog: public adevs::atomic {

public:
// creates a program with the given process ID, maximum message interval, and number
// of messages to output.  Output times are uniformly spaced between 0 and max_message_interval
	prog (unsigned pid, adevs::stime_t max_message_interval, unsigned message_count);

	void reset ();
	void delta_int ();
	void delta_ext ();
	void delta_conf ();
	void output_func ();

// number of messages in the execution list
	inline unsigned numExecuted () const { return le.getSize (); }
// returns the ith element in the execution list
	inline const message& execList (unsigned i) const { return (*(message*)le[i]); }

	~prog ();

// input port accepts a queue of messages
	static const adevs::port_t in;

// output port produces messages
	static const adevs::port_t out; 

private:

// state
	unsigned pid, to_send, sent;
	adevs::stime_t max_interval;
	message* m_out;
// the execution list
	adevs::indexed le;

};
