#include <adevs/adevs.h>
#include <stdlib.h>
#include <assert.h>
#include <iostream>
#include "host.h"
using namespace adevs;

// port definitions
const port_t host::yout = 0;
const port_t host::xout = 1;
const port_t host::yin = 2;

/**
Constructor.  Sets up the array of x values to be fed to an attached
systolic array.
*/
host::host (const double* x, unsigned length, unsigned weights):atomic () {

// set up the array of x values
this->x = (double*) malloc (sizeof (double) * length);
this->length = length;

for (unsigned i = 0; i < length; i++)
   this->x[i] = x[i];

this->weights = weights;

results = 0;
count = 0;

// add the ports
addOutputPort (yout);
addOutputPort (xout);
addInputPort (yin);

}

/**
Initialization function.
*/
void host::reset () {

count = 0;
results = 0;
// generate first output at t = 1
hold (1);

}

/**
Internal transition function
*/
void host::delta_int () { 

count++;

// iterate through array of x values, 
// generating output at 1 time unit intervals.
if (count < length) hold (1);

// passivate when all values have been sent out
else passivate ();

}

/**
External transition function.
*/
void host::delta_ext () { 

// get the values on the yin port
bag* values = inputOnPort (yin);
dbl* result = (dbl*)values->operator[](0);

// print the y value and the time of its arrival
std::cout << "host: recvd y" << results << " = " << result->value; 
std::cout << " @ t = " << timeCurrent () << std::endl;
results++;

// continue (i.e. keep tN)

}


/**
Confluent transition function.
*/
void host::delta_conf () { 

delta_int ();
delta_ext ();

}

/**
Output function
*/
void host::output_func () {

int y = count + 1 - weights;

// send an x value to the array
output (xout, new dbl (x[count]));

// send initial value of y to the array.
if (y >= 0) {
   output (yout, new dbl (0)); 
   }

}

host::~host () {

free (x);

} 
