#include <list>
#include <adevs/adevs.h>
#include <values.h>
#include "lb.h"
using namespace adevs;

const port_t lb::out = 0;
const port_t lb::in = 1;
const port_t lb::load = 2;

lb::lb (int num_procs):
atomic ("lb") {

addInputPort (in);
addInputPort (load);
addOutputPort (out);

this->num_procs = num_procs;
load_data = new int [num_procs];

}

void lb::reset () {

jobs_out.erase (jobs_out.begin(), jobs_out.end());
for (int i = 0; i < num_procs; i++)
   load_data[i] = 0;
passivate ();

}

void lb::delta_int () {

jobs_out.erase (jobs_out.begin(), jobs_out.end());
passivate ();

}

void lb::delta_ext () {

bag* values = inputOnPort (load);
for (unsigned i = 0; i < values->getSize(); i++) {
   job* j = (job*)values->getObj (i);
   load_data[j->pid]--;
   }

values = inputOnPort (in);
for (unsigned i = 0; i < values->getSize(); i++) {
   job* j = (job*)values->getObj (i);
   jobs_out.push_back (*j);
   }

assign_jobs ();
hold (0);

}

void lb::delta_conf () {

delta_int ();
delta_ext ();

}

void lb::output_func () {

std::list<job>::iterator i;
for (i = jobs_out.begin(); i != jobs_out.end(); i++)
   output (out, new job (*i));

}

lb::~lb () {

jobs_out.erase (jobs_out.begin(), jobs_out.end());
delete [] load_data;

}

void lb::assign_jobs () {

std::list<job>::iterator i;
for (i = jobs_out.begin (); i != jobs_out.end(); i++) {
   int pid = find_smallest_load ();
   load_data[pid]++;
   (*i).pid = pid;
   }

}

int lb::find_smallest_load () const {

int pid = 0;
int load = INT_MAX;

for (int i = 0; i < num_procs; i++) {
   if (load > load_data[i]) {
      load = load_data[i];
      pid = i;
      }
   }

return pid;

}
