
/**
Load balancing module.  Policy is to assign new jobs to the
processor with the shortest run list.
*/
#ifndef _LB_
#define _LB_
#include "types.h"

class lb: public adevs::atomic {

public:	
/// Create a load balancer for num_procs processors
		lb (int num_procs);

		void reset ();
		void delta_int ();
		void delta_ext ();
		void delta_conf ();
		void output_func ();

		~lb ();

/// New jobs arrive on the in port
		static const adevs::port_t in;
/// Finished jobs leave on the out port
		static const adevs::port_t out;
/// Load is reported on this port
		static const adevs::port_t load;

private:	
/// Number of processors
		int num_procs;
/// Array of load data
		int* load_data;
/// List of jobs to deliver to processors
		std::list<job> jobs_out;
/// Assign jobs based on smallest known load
		void assign_jobs ();
		int find_smallest_load () const;

};

#endif
