
/**
Transducer collects statistics about the simulation.  These
include turnaround time, throughput, number of jobs started,
and number of jobs finished.
*/

#ifndef _TRANSD_
#define _TRANSD_
#include "types.h"

typedef std::map<job, adevs::stime_t, compare_jobs> jobmap;

class transd: public adevs::atomic {

public:
/// Create a transducer that will observe the system for a given period
		transd (adevs::stime_t obs_time);

		void reset ();
		void delta_int ();
		void delta_ext ();
		void delta_conf ();
		void output_func ();
/// New jobs arrive on this port
		static const adevs::port_t ariv;
/// Finished jobs arrive on this port
		static const adevs::port_t solv;
/// Generate a stop signal when the observation period is over 
		static const adevs::port_t stop;

private:	adevs::stime_t obs_time;
		jobmap arrived;
		jobmap solved;

		void report_results ();

};

#endif
