/***********
Copyright (C) 1999, 2000  James Nutaro

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
/**

A vector in 2 dimensions.

*/

class vector {

public:

	double x, y;

// constructors
	inline vector () { x = y = 0; }
	inline vector (double x, double y) { this->x = x; this->y = y; }
	inline vector (const vector& src) { this->x = src.x; this->y = src.y; }

// returns the length of the vector
	double length () const;

// binary operators 	
	inline vector operator+ (const vector& v) const {
		return vector (v.x+x, v.y+y); } 
	inline vector operator- (const vector& v) const {
		return vector (x-v.x, y-v.y); } 
	inline vector operator* (double x) const {
		return vector (this->x*x, this->y*x); } 
	inline vector operator/ (double x) const {
		return vector (this->x/x, this->y/x); } 

// rotation and reflection transformations
	vector& rotate (double radians);
	vector& reflect (double radians);

// assignment operators
	inline vector& operator+= (const vector& v) {
		x += v.x; y += v.y; return *this; }
	inline vector& operator-= (const vector& v) {
		x -= v.x; y -= v.y; return *this; }
	inline vector& operator*= (double x) {
		this->x *= x; this->y *= x; return *this; }
	inline vector& operator/= (double x) {
		this->x /= x; this->y /= x; return *this; }
	inline vector& operator= (const vector& v) 
		{ x = v.x; y = v.y; return *this; }

// equivalence testing
	inline bool operator== (const vector& v) const {
		return (x == v.x && y == v.y); } 
	inline bool operator!= (const vector& v) const {
		return (x != v.x && y != v.y); } 

	inline ~vector () { }

};

inline vector operator* (double x, const vector& v) {
	return vector (v.x*x, v.y*x); }
