/***********
Copyright (C) 1999, 2000  James Nutaro

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
/**
The cellspace is a network executive containing a grid of cells.  The cell space
wraps at the edges.
*/

class cellspace: public adevs::networkExec {

public:		

// constructs a cellspace with the indicated width and height.
		cellspace (int width, int height);

// The compute neighhbors method is called by the simulator when messages
// appear on the output port of a cell.  Returns the cells that form the
// event generating cell's Moore neighborhood (minus the cell itself).
		const adevs::indexed* computeNeighbors (adevs::devs& src, 
				const adevs::port_value& pv);

		void reset ();
		void delta_ext ();
		void delta_int ();
		void delta_conf ();
		void output_func ();

		~cellspace ();

		static const adevs::port_t isolated;
		static const adevs::port_t birthed;

private:	
// a hashtable mapping points to cells.
		adevs::hashtable cells;
// dimensions of the cellspace.
		int width, height;
// Neighbor list returned by the computeNeighbors method.
		adevs::indexed n;

// method used to add a point to the cellspace.
		cell* add_cell (const point& loc, bool alive);
		void add_cell (cell* c);
		void remove_cell (cell* model);
// used during initialization to populate the cellspace.
		void createBlock (const point& center);
// maps x, y points into the torrodial cellspace.
		point map_point (int x, int y) const;
		void getAdj (const point& p, point* l);

};
