#include <adevs/adevs.h>
#include <iostream>
#include <assert.h>
#include "genr.h"
#include "deepgenr.h"
using namespace adevs;

/**

Simple generator test.  Targets the computeNextOutput function

**/

class _devssim: public devssim {

public:		_devssim (coupled* model);
		virtual bool halt ();

private:	int count;

};

_devssim::_devssim (coupled* model):devssim(model) { count = 0; }

bool _devssim::halt () {

indexed results;
computeNextOutput (results);

if (timeNext() == INFINITY)
   assert (results.getSize() == 0);

else {
   assert (results.getSize() == 1);
   port_value* pv = (port_value*)results[0];
   intg* i = (intg*)pv->value;
   assert (i->value == count);
   }

count++;
return devssim::halt ();

}

// start genr specification

const port_t genr::out = 0;

genr::genr ():atomic () {

period = 10.0;
tick_limit = 10;
tick_count = 0;

addOutputPort (out);

}

void genr::reset () {

tick_count = 0;
hold (0.0);

}

void genr::delta_ext () { assert (false); }

void genr::delta_int () { 

std::cout << "tick_count = " << tick_count << " @ t = " << timeCurrent () << std::endl;
tick_count++;
if (tick_count < tick_limit) {
   hold (period);
   std::cout << "period = " << period << std::endl;
   std::cout << "tN = " << timeNext () << std::endl;
   std::cout << "tL = " << timeLast () << std::endl;
   } 
else
   passivate ();

} 

void genr::delta_conf () { assert (false); }

void genr::output_func () {

output (out, new intg (tick_count));

}

genr::~genr () { }

// end of genr specification

// start deepgenr specification

const port_t deepgenr::out = 0;

deepgenr::deepgenr ():staticDigraph ("deepgenr") {

genr* g = new genr (); 

try {

add (g);
addOutputPort (out);

couple (g, g->out, this, this->out);

   }
catch (DEVSException& err) {
   std::cout << err.getError () << std::endl;
   throw err;
   }
   
}

deepgenr::~deepgenr () { }

int main () {

deepgenr* dg = new deepgenr ();

_devssim sim (dg);

try {
   sim.reset ();
   sim.run ();
   }
catch (DEVSException& err) {
   std::cout << "Caught DEVSException" << std::endl;
   std::cout << err.getError () << std::endl;
   }

}
